load_plugin 'webgen/plugins/filehandlers/filehandler'

require 'rubygems'
require 'RMagick'

MOD = self

module Slided
include MOD
  class Layouter < Webgen::Plugin

    infos( :name => 'GalleryLayouter/slides',
           :author => Webgen::AUTHOR,
           :summary => 'Handles additional tasks for the gallery layout \'slides\''
           )

    def thumbnail_for( image, data, parent )
      parent_node = @plugin_manager['File/DirectoryHandler'].recursive_create_path( File.dirname( image ), parent )
      tn_handler = @plugin_manager['File/SlidedThumbnailWriter']
      file_handler = @plugin_manager['Core/FileHandler']
      node = file_handler.create_node( File.basename( image ), parent_node, tn_handler ) do |fn, parent, h, mi|
        h.create_node( fn, parent, mi, data )
      end
      node.absolute_path
    end

    def handle_gallery( ginfo, parent, filedata )
      # Create gallery node
      gallery_thumb_writer = @plugin_manager['File/GalleryThumbnailWriter']
      file_handler = @plugin_manager['Core/FileHandler']
      g = ginfo.galleries.first

      node = file_handler.create_node(gallery_thumb_name( ginfo ), parent, gallery_thumb_writer ) do |fn, parent, h, mi|
        h.create_node( fn, parent, mi, {:parent => parent, :ginfo => ginfo, :filedata => filedata} )
      end

      ginfo[:gallery_thumb_node] = node
    end

    #######
    private
    #######

    def gallery_thumb_name( ginfo )
       ginfo['title'].tr( ' ', '_' ) + '_thumb.jpg'
    end
  end

  # Common parts for thumbnails generation
  class CommonThumbnailWriter < FileHandlers::DefaultHandler

    protected

    def get_resized( node )
      image = Magick::ImageList.new( node.node_info[:thumbnail_file] ).first
      case thumbnail_resize_method( node )
        when :cropped then cropped_thumbnail( image, thumbnail_size(node) )
        when :normal then normal_thumbnail( image, thumbnail_size(node) )
        else
          log(:warn) {"Invalid resize method specified (#{thumbnail_resize_method(node).inspect}), falling back to :normal"}
          normal_thumbnail( image, thumbnail_size(node) )
      end
      image
    end

    def normal_thumbnail( image, size )
      image.change_geometry( size ) {|c,r,i| i.resize!( c, r )}
    end

    def cropped_thumbnail( image, size )
      image.crop_resized!( *size.split('x').collect {|s| s.to_i} )
    end
  end

  # Used for creating thumbnails for images.
  class SlidedThumbnailWriter < CommonThumbnailWriter

    infos( :name => 'File/SlidedThumbnailWriter',
           :author => 'Alexander Tsvyashchenko',
           :summary => 'Writes out bordered thumbnails with RMagick'
           )

    param 'thumbnailSize', "150x150", "The size of the thumbnails"
    param 'thumbnailResizeMethod', :cropped, 'Specifies the algorithm which should be used for generating the thumbnail: ' +
      ':normal (thumbnail fits exactly into given thumbnail size), ' +
      ':cropped (resized to exact thumbnail size, image parts maybe cropped)'
    param 'slideBorderWidth', 10, 'The width of the slide border'
    param 'slideColor', '#ffffff', 'The color of the slide'
    param 'slideBorderColor', '#000000', 'The color of thumbnail border'

    def thumbnail_size( node )
      node.node_info[:filedata]['thumbnailSize'] || node['thumbnailSize'] || param( 'thumbnailSize' )
    end

    def thumbnail_resize_method( node )
      node.node_info[:filedata]['thumbnailResizeMethod'] || node['thumbnailResizeMethod'] || param( 'thumbnailResizeMethod' )
    end

    def slide_border_width( node )
      node.node_info[:filedata]['slideBorderWidth'] || node['slideBorderWidth'] || param( 'slideBorderWidth' )
    end

    def slide_color( node )
      node.node_info[:filedata]['slideColor'] || node['slideColor'] || param( 'slideColor' )
    end

    def slide_border_color( node )
      node.node_info[:filedata]['slideBorderColor'] || node['slideBorderColor'] || param( 'slideBorderColor' )
    end


    def create_node( file, parent, meta_info, data )
      node = Node.new( parent, 'tn_' + File.basename( file ).tr( ' ', '_' ) )
      node.meta_info.update( meta_info )
      node['title'] = node.path
      node.node_info[:filedata] = data[:filedata]
      node.node_info[:thumbnail_file] = file
      node.node_info[:processor] = self
      node
    end

    def write_node( node )
      if @plugin_manager['Core/FileHandler'].file_modified?( node.node_info[:thumbnail_file], node.full_path )
        log(:info) {"Creating thumbnail <#{node.full_path}> from image <#{node.node_info[:thumbnail_file]}>"}

        image = get_resized( node )

        border_width = node.slide_border_width
        width = image.columns
        height = image.rows

        thumb = image
        image = Magick::Image.new(width + 2*border_width, height + 2*border_width)
        image.background_color = 'transparent'
        gc = Magick::Draw.new
        gc.fill_opacity(0)
        gc.stroke( node.slide_color )
        gc.stroke_width( node.slide_border_width )
        gc.roundrectangle( border_width/2, border_width/2, width + 3*border_width/2, height + 3*border_width/2, border_width + 1, border_width + 1)
        image.composite!(thumb, border_width, border_width, Magick::OverCompositeOp)
        gc.stroke_width(1)
        gc.stroke( node.slide_border_color )
        gc.rectangle( border_width + 1, border_width + 1, width + border_width - 1, height + border_width - 1)
        gc.draw(image)

        image.write( node.full_path )
      end
    end
  end

  class GalleryThumbnailWriter < CommonThumbnailWriter

    infos( :name => 'File/GalleryThumbnailWriter',
           :author => 'Alexander Tsvyashchenko',
           :summary => 'Generates a special thumbnail suitable for gallery folder image' )

    param 'thumbnailGallerySize', "150x150", "The size of the thumbnails"
    param 'thumbnailGalleryResizeMethod', :cropped, 'Specifies the algorithm which should be used for generating the thumbnail: ' +
      ':normal (thumbnail fits exactly into given thumbnail size), ' +
      ':cropped (resized to exact thumbnail size, image parts maybe cropped)'
    param 'image', nil, 'The image to use for thumbnail'
    param 'galleryColor', '#ffffff', 'The color of the gallery thumbnail border'
    param 'galleryBorderColor', '#000000', 'The color of thumbnail border'
    param 'galleryImageTemplate', nil, 'The image to use as template'

    def thumbnail_size( node )
      node.node_info[:filedata]['thumbnailGallerySize'] || node['thumbnailGallerySize'] || param( 'thumbnailGallerySize' )
    end

    def thumbnail_resize_method( node )
      node.node_info[:filedata]['thumbnailGalleryResizeMethod'] || node['thumbnailResizeMethod'] || param( 'thumbnailGalleryResizeMethod' )
    end

    def gallery_color( node )
      node.node_info[:filedata]['galleryColor'] || node['galleryColor'] || param( 'galleryColor' )
    end

    def gallery_border_color( node )
      node.node_info[:filedata]['galleryBorderColor'] || node['galleryBorderColor'] || param( 'galleryBorderColor' )
    end

    def gallery_image_template( node )
      node.node_info[:filedata]['galleryImageTemplate'] || node['galleryImageTemplate'] || param( 'galleryImageTemplate' )
    end

    def thumbnail_image( node )
      gal_writer = node.node_info[:data]
      if node.node_info[:filedata]['File/GalleryThumbnailWriter']
        File.dirname(node.full_path) + '/' + node.node_info[:filedata]['File/GalleryThumbnailWriter']['image']
      else
        File.dirname(node.full_path) + '/' + node.node_info[:ginfo].galleries.first.images.first.filename
      end
    end

    def create_node( path, parent, meta_info, data )
      node = Node.new( parent, File.basename( path ) )
      node.meta_info.update( meta_info )
      node.node_info[:filedata] = data[:filedata][0]
      node.node_info[:ginfo] = data[:ginfo]
      node.node_info[:thumbnail_file] = thumbnail_image( node )
      node.node_info[:processor] = self
      node
    end

    def write_node( node )
      if @plugin_manager['Core/FileHandler'].file_modified?( node.node_info[:thumbnail_file], node.full_path )
        thumb = get_resized( node )

        width = thumb.columns
        height = thumb.rows

        # Starting positions of thumbnail image on template image
        x_start = 22
        y_start = 24
        # Width of border around thumbnail image
        border_width = 10
        # Width of slide shadows
        shadow_width = 2
        # Maximal thumbnail sizes the image is designed for
        thumb_width = 75
        thumb_height = 75

        image = Magick::Image.new(x_start + width + 2*border_width + shadow_width, y_start + height + 2*border_width)

        img_tmpl = Magick::Image.read('F:/_dev/projects/homepage/plugin/folder1.png').first
        tmpl_width = img_tmpl.columns
        tmpl_height = img_tmpl.rows

        comb_height = y_start + height + 2*border_width
        main_img = img_tmpl.crop(0, tmpl_height - comb_height, x_start + width + 2*border_width, comb_height)
        lu_cnr = img_tmpl.crop(x_start, 0, border_width, border_width)
        half_height = height / 2 + border_width
        ru_cnr = img_tmpl.crop(x_start + thumb_width + border_width + 1, 0, border_width + shadow_width, half_height + 1)
        rd_cnr = img_tmpl.crop(x_start + thumb_width + border_width + 1, thumb_height + 2*border_width - half_height + 1, border_width + shadow_width, half_height + shadow_width)

        image.composite!(main_img, 0, 0, Magick::OverCompositeOp)
        image.composite!(thumb, x_start + border_width, border_width, Magick::OverCompositeOp)

        image.composite!(lu_cnr, x_start, 0, Magick::OverCompositeOp)
        image.composite!(ru_cnr, x_start + width + border_width, 0, Magick::OverCompositeOp)
        image.composite!(rd_cnr, x_start + width + border_width, height + 2*border_width - half_height, Magick::OverCompositeOp)

        image.background_color = 'transparent'
        gc = Magick::Draw.new
        gc.fill_opacity(0)
        gc.stroke( node.gallery_border_color )
        gc.stroke_width(1)
        gc.rectangle( x_start + border_width, border_width, x_start + border_width + width - 1, border_width + height - 1)
        gc.draw(image)

        image.write( node.full_path )
      end
    end
  end
end
